from django.conf.urls import url

from . import views

urlpatterns = [
    url('^save/$', views.save, name='usermedia_save'),
    url('^delete/$', views.delete, name='usermedia_delete'),
    url('^images/$', views.images, name='usermedia_images'),
    url(
        '^save_category/$',
        views.save_category,
        name='usermedia_save_category'
    )
]



from django.contrib import admin
from usermedia.models import Image, UserImage, ImageCategory, DocumentImage


class ImageAdmin(admin.ModelAdmin):
    pass


admin.site.register(Image, ImageAdmin)


class UserImageAdmin(admin.ModelAdmin):
    pass


admin.site.register(UserImage, UserImageAdmin)


class DocumentImageAdmin(admin.ModelAdmin):
    pass


admin.site.register(DocumentImage, DocumentImageAdmin)


class ImageCategoryAdmin(admin.ModelAdmin):
    pass


admin.site.register(ImageCategory, ImageCategoryAdmin)



import shutil
from subprocess import call
from base.management import BaseCommand
from django.conf import settings


class Command(BaseCommand):
    help = 'Check Python files for PEP8 compliance'

    def handle(self, *args, **options):
        self.stdout.write("Linting Python code...")
        shutil.os.chdir(settings.PROJECT_PATH)
        return_value = call([
            "flake8",
            "--exclude",
            "node_modules/*,*/migrations/*,venv/*,.direnv/*,configuration.py",
            "./"
        ])
        if return_value > 0:
            exit(return_value)



import json
from django.db import migrations, models


def text_to_json(apps, schema_editor):
    UserImage = apps.get_model('usermedia', 'UserImage')
    uimages = UserImage.objects.all().iterator()
    for image in uimages:
        try:
            image.cats = json.loads(image.image_cat)
        except ValueError:
            pass
        try:
            image.copyright = json.loads(image.copyright_text)
        except ValueError:
            image.copyright = {
                "holder": False,
                "year": False,
                "freeToRead": True,
                "licenses": []
            }
        image.save()
    DocumentImage = apps.get_model('usermedia', 'DocumentImage')
    dimages = DocumentImage.objects.all().iterator()
    for image in dimages:
        try:
            image.copyright = json.loads(image.copyright_text)
        except ValueError:
            image.copyright = {
                "holder": False,
                "year": False,
                "freeToRead": True,
                "licenses": []
            }
        image.save()

def json_to_text(apps, schema_editor):
    UserImage = apps.get_model('usermedia', 'UserImage')
    images = UserImage.objects.all().iterator()
    for image in images:
        image.image_cat = json.dumps(image.cats)
        image.copyright_text = json.dumps(image.copyright)
        image.save()
    DocumentImage = apps.get_model('usermedia', 'DocumentImage')
    dimages = DocumentImage.objects.all().iterator()
    for image in dimages:
        image.copyright_text = json.dumps(image.copyright)
        image.save()


class Migration(migrations.Migration):

    dependencies = [
        ('usermedia', '0004_auto_20200205_2347'),
    ]

    operations = [
        migrations.AddField(
            model_name='userimage',
            name='cats',
            field=models.JSONField(default=list),
        ),
        migrations.RenameField(
            model_name='userimage',
            old_name='copyright',
            new_name='copyright_text'
        ),
        migrations.AddField(
            model_name='userimage',
            name='copyright',
            field=models.JSONField(default=dict),
        ),
        migrations.RenameField(
            model_name='documentimage',
            old_name='copyright',
            new_name='copyright_text'
        ),
        migrations.AddField(
            model_name='documentimage',
            name='copyright',
            field=models.JSONField(default=dict),
        ),
        migrations.RunPython(text_to_json, json_to_text),
        migrations.RemoveField(
            model_name='userimage',
            name='image_cat',
        ),
        migrations.RemoveField(
            model_name='userimage',
            name='copyright_text',
        ),
        migrations.RemoveField(
            model_name='documentimage',
            name='copyright_text',
        ),
    ]



from builtins import object
from django.db import models
from django.contrib.auth.models import User


class UserProfile(models.Model):
    user = models.OneToOneField(
        User,
        on_delete=models.deletion.CASCADE
    )
    about = models.TextField(max_length=500, blank=True)


User.profile = property(lambda u: UserProfile.objects.get_or_create(user=u)[0])


def get_readable_name(user):
    readable_name = user.get_full_name()
    if readable_name == '':
        readable_name = user.username
    return readable_name


User.readable_name = property(lambda u: get_readable_name(u))


# Anyone can define anyone as a team member without approval from that person.
# This works in only one direction. Team leaders are the owners of their
# documents, and the same user can be a Team member on the document of someone
# else.
# Students/academics helping oneanother with their writings with shifting
# responsibilities are the use case in mind here.
# Roles are stored as a comma separated list as a CharField


class TeamMember(models.Model):
    leader = models.ForeignKey(
        User,
        related_name='leader',
        on_delete=models.deletion.CASCADE
    )
    member = models.ForeignKey(
        User,
        related_name='member',
        on_delete=models.deletion.CASCADE
    )
    roles = models.CharField(max_length=100, blank=True)

    class Meta(object):
        unique_together = (("leader", "member"),)



from django.conf.urls import url

from . import views

urlpatterns = [
    url('^list/$', views.list, name='document_template_list'),
    url('^save/$', views.save, name='document_template_save'),
    url('^get/$', views.get, name='document_template_get'),
    url('^copy/$', views.copy, name='document_template_copy'),
    url('^delete/$', views.delete, name='document_template_delete'),
]



from urllib.parse import urlparse
from tornado.websocket import WebSocketHandler
from tornado.websocket import WebSocketClosedError
from tornado.iostream import StreamClosedError
import tornado
from django.db import connection
import logging
from logging import info, debug
from tornado.ioloop import IOLoop
from tornado.escape import json_decode

from .django_handler_mixin import DjangoHandlerMixin

logger = logging.getLogger(__name__)


class BaseWebSocketHandler(DjangoHandlerMixin, WebSocketHandler):
    def initialize(self, app_name):
        self.app_name = app_name

    def open(self, arg):
        self.set_nodelay(True)
        logger.debug("Action:Opening Websocket")
        self.id = 0
        self.user = self.get_current_user()
        self.endpoint = self.app_name+'/'+arg
        self.args = arg.split("/")
        self.messages = {
            'server': 0,
            'client': 0,
            'last_ten': []
        }
        if not self.user.is_authenticated:
            self.access_denied()
            return
        logger.debug(
            f"Action:Opening Websocket URL:{self.endpoint}"
            f" User:{self.user.id} ParticipantID:{self.id}")
        response = dict()
        response['type'] = 'welcome'
        self.send_message(response)

    def access_denied(self):
        response = dict()
        response['type'] = 'access_denied'
        self.send_message(response)
        IOLoop.current().add_callback(self.do_close)
        return

    def do_close(self):
        self.close()

    def on_message(self, data):
        message = json_decode(data)
        if message["type"] == 'request_resend':
            self.resend_messages(message["from"])
            return
        if 'c' not in message and 's' not in message:
            self.send({
                'type': 'access_denied'
            })
            # Message doesn't contain needed client/server info. Ignore.
            return
        logger.debug(
            f"Action:Message received URL:{self.endpoint} "
            f"User:{self.user.id} ParticipantID:{self.id} "
            f"Type:{message['type']} "
            f"S count client:{message['s']} C count client:{message['c']} "
            f"S count server:{self.messages['server']} "
            f"C count server:{self.messages['client']}")

        if message["c"] < (self.messages["client"] + 1):
            # Receive a message already received at least once. Ignore.
            return
        elif message["c"] > (self.messages["client"] + 1):
            # Messages from the client have been lost.
            logger.debug(
                f"Action:Requesting resending of lost messages from client"
                f" URL:{self.endpoint} User:{self.user.id} "
                f"ParticipantID:{self.id} from:{self.messages['client']}")

            self.send({
                'type': 'request_resend',
                'from': self.messages["client"]
            })
            return
        elif message["s"] < self.messages["server"]:
            # Message was sent either simultaneously with message from server
            # or a message from the server previously sent never arrived.
            # Resend the messages the client missed.
            logger.debug(
                f"Action:Simultaneous.Resend messages to client. "
                f"URL:{self.endpoint} User:{self.user.id} "
                f"ParticipantID:{self.id} from:{message['s']}")

            self.messages["client"] += 1
            self.resend_messages(message["s"])
            self.reject_message(message)
            return
        # Message order is correct. We continue processing the data.
        self.messages["client"] += 1
        self.handle_message(message)

    def handle_message(message):
        pass

    def reject_message(message):
        pass

    def send_message(self, message):
        self.messages['server'] += 1
        message['c'] = self.messages['client']
        message['s'] = self.messages['server']
        self.messages['last_ten'].append(message)
        self.messages['last_ten'] = self.messages['last_ten'][-10:]
        logger.debug(
            f"Action:Sending Message. URL:{self.endpoint} User:{self.user.id} "
            f"ParticipantID:{self.id} Type:{message['type']} "
            f"S count server:{message['s']} C count server:{message['c']}")
        self.send(message)

    @tornado.gen.coroutine
    def send(self, message):
        try:
            yield self.write_message(message)
        except (WebSocketClosedError, StreamClosedError):
            pass

    def unfixable(self):
        pass

    def resend_messages(self, from_no):
        to_send = self.messages["server"] - from_no
        logger.debug(
            f"Action:Resending messages to User. URL:{self.endpoint} "
            f"User:{self.user.id} ParticipantID:{self.id} "
            f"number of messages to be resent:{to_send} "
            f"S count server:{self.messages['server']} from:{from_no}")
        self.messages['server'] -= to_send
        if to_send > len(self.messages['last_ten']):
            # Too many messages requested. We have to abort.
            logger.debug(
                f"Action:Lot of messages requested. URL:{self.endpoint} "
                f"User:{self.user.id} ParticipantID:{self.id} "
                f"number of messages requested:{to_send}")
            self.unfixable()
            return
        for message in self.messages['last_ten'][0-to_send:]:
            self.send_message(message)

    def check_origin(self, origin):
        parsed_origin = urlparse(origin)
        origin = parsed_origin.netloc
        # remove port if present
        origin = origin.split(':')[0]
        origin = origin.lower()

        host = self.request.headers.get("Host")
        # remove port if present
        host = host.split(':')[0]
        # Check to see that origin matches host directly, EXCLUDING ports
        return origin == host

    def prepare(self):
        super(BaseWebSocketHandler, self).prepare()

    def finish(self, chunk=None):
        super(BaseWebSocketHandler, self).finish(chunk=chunk)

    # Clean up django ORM connections

        connection.close()
        if False:
            info('%d sql queries' % len(connection.queries))
            for query in connection.queries:
                debug('%s [%s seconds]' % (query['sql'], query['time']))

    # Clean up after python-memcached

        from django.core.cache import cache
        if hasattr(cache, 'close'):
            cache.close()



import time
import os

from testing.testcases import LiveTornadoTestCase
from testing.selenium_helper import SeleniumHelper
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.action_chains import ActionChains

from django.core import mail
from django.conf import settings

from allauth.account.models import EmailConfirmationHMAC, EmailAddress


class EditorTest(LiveTornadoTestCase, SeleniumHelper):
    fixtures = [
        'initial_documenttemplates.json',
        'initial_styles.json',
    ]

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.base_url = cls.live_server_url
        driver_data = cls.get_drivers(1)
        cls.driver = driver_data["drivers"][0]
        cls.client = driver_data["clients"][0]
        cls.driver.implicitly_wait(driver_data["wait_time"])
        cls.wait_time = driver_data["wait_time"]

    @classmethod
    def tearDownClass(cls):
        cls.driver.quit()
        super().tearDownClass()

    def setUp(self):
        self.verificationErrors = []
        self.accept_next_alert = True
        self.user1 = self.create_user(
            username='Yeti',
            email='yeti@snowman.com',
            passtext='otter'
        )

    def tearDown(self):
        self.leave_site(self.driver)

    def test_crossrefs_and_internal_links(self):
        self.driver.get(self.base_url)
        self.driver.find_element(By.ID, "id_login").send_keys("Yeti")
        self.driver.find_element(By.ID, "id_password").send_keys("otter")
        self.driver.find_element(By.ID, "login-submit").click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable(
                (
                    By.CSS_SELECTOR,
                    ".new_document button"
                )
            )
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located((By.CLASS_NAME, 'editor-toolbar'))
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").send_keys(
            "Test"
        )
        # We enable the abstract
        self.driver.find_element(
            By.CSS_SELECTOR,
            "#header-navigation > div:nth-child(3) > span"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            (
                "#header-navigation > div:nth-child(3) > div "
                "> ul > li:nth-child(1) > span"
            )
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            (
                "#header-navigation > div:nth-child(3) > div "
                "> ul > li:nth-child(1) > div > ul > li:nth-child(3) > span"
            )
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").click()
        ActionChains(self.driver).send_keys(
            Keys.LEFT
        ).send_keys(
            "An abstract title"
        ).perform()
        time.sleep(1)
        self.driver.find_element(
            By.CSS_SELECTOR,
            "#toolbar > div > div > div:nth-child(3) > div"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            (
                "#toolbar > div > div > div:nth-child(3) > div > div > "
                "ul > li:nth-child(4) > span > label"
            )
        ).click()
        # We type in the body
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "Body"
        )
        # We add a figure
        button = self.driver.find_element_by_xpath('//*[@title="Figure"]')
        button.click()

        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (By.CSS_SELECTOR, "span.math-field")
            )
        )
        self.driver.find_element_by_css_selector(
            "div.figure-category"
        ).click()
        self.driver.find_element_by_xpath(
            '//*[normalize-space()="Photo"]'
        ).click()

        # click on 'Insert image' button
        self.driver.find_element_by_id('insert-figure-image').click()

        upload_button = WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (
                    By.XPATH,
                    '//*[normalize-space()="Add new image"]'
                )
            )
        )

        upload_button.click()

        # image path
        image_path = os.path.join(
            settings.PROJECT_PATH,
            'document/tests/uploads/image.png'
        )

        # in order to select the image we send the image path in the
        # LOCAL MACHINE to the input tag
        upload_image_url = WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (By.XPATH, '//*[@id="editimage"]/div[1]/input[2]')
            )
        )
        upload_image_url.send_keys(image_path)

        # click on 'Upload' button
        self.driver.find_element_by_xpath(
            '//*[contains(@class, "ui-button") and normalize-space()="Upload"]'
        ).click()

        # click on 'Use image' button
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable(
                (By.CSS_SELECTOR, '.fw-data-table i.fa-check')
            )
        )

        self.driver.find_element_by_xpath(
            '//*[normalize-space()="Use image"]'
        ).click()
        self.driver.find_element_by_css_selector("button.fw-dark").click()

        caption = WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (By.CSS_SELECTOR, "div.article-body figure figcaption")
            )
        )

        caption.click()

        caption.send_keys("Caption")

        ActionChains(self.driver).send_keys(
            Keys.RIGHT
        ).perform()
        # We add a cross reference for the heading
        self.driver.find_element(
            By.CSS_SELECTOR,
            "#toolbar > div > div > div:nth-child(9) > button > span > i"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "#edit-link > div:nth-child(2) > select"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "#edit-link > div:nth-child(2) > select > option:nth-child(2)"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            (
                "body > div.ui-dialog.ui-corner-all.ui-widget."
                "ui-widget-content.ui-front.ui-dialog-buttons > "
                "div.ui-dialog-buttonpane.ui-widget-content."
                "ui-helper-clearfix > div > button.fw-dark."
                "fw-button.ui-button.ui-corner-all.ui-widget"
            )
        ).click()
        cross_reference = self.driver.find_element(
            By.CSS_SELECTOR,
            ".article-body .cross-reference"
        )
        assert cross_reference.text == 'An abstract title'
        # We add a second cross reference to the figure
        self.driver.find_element(
            By.CSS_SELECTOR,
            "#toolbar > div > div > div:nth-child(9) > button > span > i"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "#edit-link > div:nth-child(2) > select"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "#edit-link > div:nth-child(2) > select > option:nth-child(3)"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            (
                "body > div.ui-dialog.ui-corner-all.ui-widget."
                "ui-widget-content.ui-front.ui-dialog-buttons > "
                "div.ui-dialog-buttonpane.ui-widget-content."
                "ui-helper-clearfix > div > button.fw-dark."
                "fw-button.ui-button.ui-corner-all.ui-widget"
            )
        ).click()
        figure_cross_reference = self.driver.find_elements(
            By.CSS_SELECTOR,
            ".article-body .cross-reference"
        )[1]
        assert figure_cross_reference.text == 'Photo 1'
        # We add an internal link
        self.driver.find_element(
            By.CSS_SELECTOR,
            "#toolbar > div > div > div:nth-child(9) > button > span > i"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "#edit-link > div:nth-child(5) > select"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "#edit-link > div:nth-child(5) > select > option:nth-child(2)"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            (
                "body > div.ui-dialog.ui-corner-all.ui-widget."
                "ui-widget-content.ui-front.ui-dialog-buttons > "
                "div.ui-dialog-buttonpane.ui-widget-content."
                "ui-helper-clearfix > div > button.fw-dark."
                "fw-button.ui-button.ui-corner-all.ui-widget"
            )
        ).click()
        internal_link = self.driver.find_element(
            By.CSS_SELECTOR,
            ".article-body a"
        )
        assert internal_link.text == 'An abstract title'
        # We change the link text.
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".article-abstract h3"
        ).click()
        ActionChains(self.driver).send_keys(
            Keys.BACKSPACE
        ).send_keys(
            Keys.BACKSPACE
        ).send_keys(
            Keys.BACKSPACE
        ).send_keys(
            Keys.BACKSPACE
        ).send_keys(
            Keys.BACKSPACE
        ).send_keys(
            Keys.BACKSPACE
        ).perform()
        internal_link = self.driver.find_element(
            By.CSS_SELECTOR,
            ".article-body a"
        )
        assert internal_link.text == 'An abstract title'
        assert internal_link.get_attribute("title") == 'An abstract'
        cross_reference = self.driver.find_element(
            By.CSS_SELECTOR,
            ".article-body .cross-reference"
        )
        assert cross_reference.text == 'An abstract'
        # We add a second photo figure to increase the count
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".article-body figure"
        ).click()
        ActionChains(self.driver).send_keys(
            Keys.LEFT
        ).perform()
        button = self.driver.find_element_by_xpath('//*[@title="Figure"]')
        button.click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (By.CSS_SELECTOR, "span.math-field")
            )
        )
        self.driver.find_element_by_css_selector(
            "div.figure-category"
        ).click()
        self.driver.find_element_by_xpath(
            '//*[normalize-space()="Photo"]'
        ).click()

        # click on 'Insert image' button
        self.driver.find_element_by_id('insert-figure-image').click()
        # click on 'Use image' button
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable(
                (By.CSS_SELECTOR, '.dataTable-container img')
            )
        ).click()

        self.driver.find_element_by_xpath(
            '//*[normalize-space()="Use image"]'
        ).click()
        self.driver.find_element_by_css_selector("button.fw-dark").click()
        time.sleep(1)
        figure_cross_reference = self.driver.find_elements(
            By.CSS_SELECTOR,
            ".article-body .cross-reference"
        )[1]
        assert figure_cross_reference.text == 'Photo 2'

        # We delete the contents from the heading
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".article-abstract h3"
        ).click()
        ActionChains(self.driver).send_keys(
            Keys.BACKSPACE
        ).send_keys(
            Keys.BACKSPACE
        ).send_keys(
            Keys.BACKSPACE
        ).send_keys(
            Keys.BACKSPACE
        ).send_keys(
            Keys.BACKSPACE
        ).send_keys(
            Keys.BACKSPACE
        ).send_keys(
            Keys.BACKSPACE
        ).send_keys(
            Keys.BACKSPACE
        ).send_keys(
            Keys.BACKSPACE
        ).send_keys(
            Keys.BACKSPACE
        ).send_keys(
            Keys.BACKSPACE
        ).perform()
        cross_reference = self.driver.find_element(
            By.CSS_SELECTOR,
            ".article-body .cross-reference.missing-target"
        )
        assert cross_reference.text == 'MISSING TARGET'
        internal_link = self.driver.find_element(
            By.CSS_SELECTOR,
            ".article-body a.missing-target"
        )
        assert internal_link.get_attribute("title") == 'Missing target'
        self.assertEqual(
            len(self.driver.find_elements_by_css_selector(
                '.margin-box.warning'
            )),
            2
        )
        # We add text to the heading again
        ActionChains(self.driver).send_keys(
            'Title'
        ).perform()
        cross_reference = self.driver.find_element(
            By.CSS_SELECTOR,
            ".article-body .cross-reference"
        )
        assert cross_reference.text == 'Title'
        internal_link = self.driver.find_element(
            By.CSS_SELECTOR,
            ".article-body a"
        )
        assert internal_link.get_attribute("title") == 'Title'
        self.assertEqual(
            len(self.driver.find_elements_by_css_selector(
                '.margin-box.warning'
            )),
            0
        )
        # We remove the second figure
        self.driver.find_elements(
            By.CSS_SELECTOR,
            ".article-body figure"
        )[1].click()
        button = self.driver.find_element_by_xpath('//*[@title="Figure"]')
        button.click()
        self.driver.find_element_by_xpath(
            '//*[contains(@class, "ui-button") and normalize-space()="Remove"]'
        ).click()
        time.sleep(1)
        figure_cross_reference = self.driver.find_elements(
            By.CSS_SELECTOR,
            ".article-body .cross-reference"
        )[1]
        assert figure_cross_reference.text == 'MISSING TARGET'

    def check_body(self, driver, body_text, seconds=False):
        if seconds is False:
            seconds = self.wait_time
        # Contents is child 5.
        current_body_text = driver.execute_script(
            'return window.theApp.page.view.state.doc.firstChild'
            '.child(5).textContent;'
        )
        if seconds < 0:
            assert False, "Body text incorrect: {}".format(current_body_text)
        elif current_body_text == body_text:
            return True
        else:
            time.sleep(0.1)
            return self.check_body(driver, body_text, seconds - 0.1)

    def test_track_changes(self):
        self.driver.get(self.base_url)
        self.driver.find_element(By.ID, "id_login").send_keys("Yeti")
        self.driver.find_element(By.ID, "id_password").send_keys("otter")
        self.driver.find_element(By.ID, "login-submit").click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable(
                (
                    By.CSS_SELECTOR,
                    ".new_document button"
                )
            )
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located((By.CLASS_NAME, 'editor-toolbar'))
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").send_keys(
            "A test article with tracked changes"
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "First I type "
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            "button[title=Strong]"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "some"
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            "button[title=Strong]"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            " standard "
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            "button[title=Emphasis]"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "text"
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            "button[title=Emphasis]"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            " here.\nI'll even write a second paragraph."
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".header-menu:nth-child(5) > .header-nav-item"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "li:nth-child(1) > .fw-pulldown-item"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").click()
        ActionChains(self.driver).double_click(
            self.driver.find_element(By.CSS_SELECTOR, ".article-body strong")
        ).perform()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "button[title=Strong]"
        ).click()
        ActionChains(self.driver).double_click(
            self.driver.find_element(By.CSS_SELECTOR, ".article-body em")
        ).perform()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "button[title=Strong]"
        ).click()
        ActionChains(self.driver).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.BACKSPACE
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            "insertion"
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.ENTER
        ).send_keys(
            Keys.UP
        ).perform()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".editor-toolbar .multiButtons"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".editor-toolbar .multiButtons .ui-button:nth-child(5)"
        ).click()

        change_tracking_boxes = self.driver.find_elements_by_css_selector(
            '.margin-box.track'
        )
        self.assertEqual(
            len(change_tracking_boxes),
            6
        )

    def test_share_document(self):
        self.create_user(
            username='Yeti2',
            email='yeti2@snowman.com',
            passtext='otter'
        )
        self.driver.get(self.base_url)
        self.driver.find_element(By.ID, "id_login").send_keys("Yeti")
        self.driver.find_element(By.ID, "id_password").send_keys("otter")
        self.driver.find_element(By.ID, "login-submit").click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable(
                (
                    By.CSS_SELECTOR,
                    ".new_document button"
                )
            )
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located((By.CLASS_NAME, 'editor-toolbar'))
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").send_keys(
            "A test article to share"
        )
        # Open share dialog
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".header-menu:nth-child(1) > .header-nav-item"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "li:nth-child(1) > .fw-pulldown-item"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".ui-dialog .fw-add-button"
        ).click()
        self.driver.find_element(
            By.ID,
            "new-member-user-string"
        ).click()
        self.driver.find_element(By.ID, "new-member-user-string").send_keys(
            "yeti2@snowman.com"
        )
        ActionChains(self.driver).send_keys(
            Keys.TAB
        ).send_keys(
            Keys.RETURN
        ).perform()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".ui-dialog .fw-add-button"
        ).click()
        self.driver.find_element(
            By.ID,
            "new-member-user-string"
        ).click()
        self.driver.find_element(By.ID, "new-member-user-string").send_keys(
            "yeti3@snowman.com"
        )
        ActionChains(self.driver).send_keys(
            Keys.TAB
        ).send_keys(
            Keys.RETURN
        ).perform()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".ui-dialog .fw-add-button"
        ).click()
        self.driver.find_element(
            By.ID,
            "new-member-user-string"
        ).click()
        self.driver.find_element(By.ID, "new-member-user-string").send_keys(
            "yeti4@snowman.com"
        )
        ActionChains(self.driver).send_keys(
            Keys.TAB
        ).send_keys(
            Keys.RETURN
        ).perform()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable(
                (By.CSS_SELECTOR, ".collaborator-tr .fa-caret-down")
            )
        ).click()
        self.driver.find_element_by_xpath(
            '//*[normalize-space()="Write"]'
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "#my-contacts"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "tr:nth-child(3) .fa-caret-down"
        ).click()
        self.driver.find_element_by_xpath(
            '//*[normalize-space()="Write"]'
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "#my-contacts"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".ui-dialog .fw-dark"
        ).click()
        time.sleep(1)
        # We keep track of the invitation email to open it later.
        user4_invitation_email = mail.outbox[-1].body
        #  Reopen the share dialog and add a fifth user
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".header-menu:nth-child(1) > .header-nav-item"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "li:nth-child(1) > .fw-pulldown-item"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".ui-dialog .fw-add-button"
        ).click()
        self.driver.find_element(
            By.ID,
            "new-member-user-string"
        ).click()
        self.driver.find_element(By.ID, "new-member-user-string").send_keys(
            "yeti5@snowman.com"
        )
        ActionChains(self.driver).send_keys(
            Keys.TAB
        ).send_keys(
            Keys.RETURN
        ).perform()
        # Downgrade the write rights to read rights for user4
        self.driver.find_element(
            By.CSS_SELECTOR,
            "tr:nth-child(3) .fa-caret-down.edit-right"
        ).click()
        self.driver.find_element_by_xpath(
            '//*[normalize-space()="Read"]'
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "#my-contacts"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".ui-dialog .fw-dark"
        ).click()
        time.sleep(1)
        # We keep track of the invitation email to open it later.
        user5_invitation_email = mail.outbox[-1].body
        # We close the editor
        self.driver.find_element(
            By.ID,
            "close-document-top"
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.ID, 'preferences-btn'))
        ).click()
        self.driver.find_element_by_xpath(
            '//*[normalize-space()="Log out"]'
        ).click()
        # Second user logs in, verifies that he has access
        self.driver.find_element(By.ID, "id_login").send_keys("Yeti2")
        self.driver.find_element(By.ID, "id_password").send_keys("otter")
        self.driver.find_element(By.ID, "login-submit").click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (
                    By.CSS_SELECTOR,
                    ".new_document button"
                )
            )
        )
        documents = self.driver.find_elements_by_css_selector(
            '.fw-contents tbody tr'
        )
        self.assertEqual(
            len(documents),
            1
        )
        write_access_rights = self.driver.find_elements_by_css_selector(
            '.fw-contents tbody tr .icon-access-write'
        )
        self.assertEqual(
            len(write_access_rights),
            1
        )
        self.driver.find_element_by_css_selector(
            '.fw-contents tbody tr a.doc-title'
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located((By.CLASS_NAME, 'editor-toolbar'))
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").click()
        ActionChains(self.driver).send_keys(
            "The body"
        ).send_keys(
            Keys.ENTER
        ).perform()
        time.sleep(1)
        assert self.driver.find_element(
            By.CSS_SELECTOR,
            ".article-title"
        ).text == "A test article to share"
        self.check_body(self.driver, 'The body')
        self.driver.find_element(
            By.ID,
            "close-document-top"
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.ID, 'preferences-btn'))
        ).click()
        self.driver.find_element_by_xpath(
            '//*[normalize-space()="Log out"]'
        ).click()
        # First user logs in again, removes access rights of second user
        self.driver.find_element(By.ID, "id_login").send_keys("Yeti")
        self.driver.find_element(By.ID, "id_password").send_keys("otter")
        self.driver.find_element(By.ID, "login-submit").click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (
                    By.CSS_SELECTOR,
                    ".new_document button"
                )
            )
        )
        documents = self.driver.find_elements_by_css_selector(
            '.fw-contents tbody tr a.doc-title'
        )
        self.assertEqual(
            len(documents),
            1
        )
        self.driver.find_element_by_css_selector(
            '.fw-contents tbody tr a.doc-title'
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located((By.CLASS_NAME, 'editor-toolbar'))
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".header-menu:nth-child(1) > .header-nav-item"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "li:nth-child(1) > .fw-pulldown-item"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".delete-collaborator"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".ui-dialog .fw-dark"
        ).click()
        self.driver.find_element(
            By.ID,
            "close-document-top"
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.ID, 'preferences-btn'))
        ).click()
        self.driver.find_element_by_xpath(
            '//*[normalize-space()="Log out"]'
        ).click()
        # Second user logs in again to verify that access rights are gone
        self.driver.find_element(By.ID, "id_login").send_keys("Yeti2")
        self.driver.find_element(By.ID, "id_password").send_keys("otter")
        self.driver.find_element(By.ID, "login-submit").click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (
                    By.CSS_SELECTOR,
                    ".new_document button"
                )
            )
        )
        documents = self.driver.find_elements_by_css_selector(
            '.fw-contents tbody tr a.doc-title'
        )
        self.assertEqual(
            len(documents),
            0
        )
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.ID, 'preferences-btn'))
        ).click()
        self.driver.find_element_by_xpath(
            '//*[normalize-space()="Log out"]'
        ).click()
        # Third user signs up
        self.driver.find_element(
            By.CSS_SELECTOR,
            'a[title="Sign up"]'
        ).click()
        self.driver.find_element(
            By.ID,
            'id_username'
        ).send_keys('Yeti3')
        self.driver.find_element(
            By.ID,
            'id_password1'
        ).send_keys('password')
        self.driver.find_element(
            By.ID,
            'id_password2'
        ).send_keys('password')
        self.driver.find_element(
            By.ID,
            'id_email'
        ).send_keys('yeti3@snowman.com')
        # Scroll all the way down in case we are on a small screen
        self.driver.execute_script(
            "window.scrollTo(0, document.body.scrollHeight);"
        )
        self.driver.find_element(
            By.ID,
            'signup-submit'
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (
                    By.CSS_SELECTOR,
                    'a[href="mailto:yeti3@snowman.com"]'
                )
            )
        )
        email = EmailAddress.objects.filter(email='yeti3@snowman.com').first()
        self.assertIsNotNone(email)
        confirmation_key = EmailConfirmationHMAC(email).key
        self.driver.get(
            self.base_url +
            "/account/confirm-email/" +
            confirmation_key +
            "/"
        )
        self.driver.find_element(
            By.ID,
            'terms-check'
        ).click()
        self.driver.find_element(
            By.ID,
            'test-check'
        ).click()
        self.driver.find_element(
            By.ID,
            'submit'
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.text_to_be_present_in_element(
                (
                    By.CSS_SELECTOR,
                    ".fw-contents h1"
                ),
                "Thanks for verifying!"
            )
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            'a[href="/"]'
        ).click()
        self.driver.find_element(By.ID, "id_login").send_keys("Yeti3")
        self.driver.find_element(By.ID, "id_password").send_keys("password")
        self.driver.find_element(By.ID, "login-submit").click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (
                    By.CSS_SELECTOR,
                    ".new_document button"
                )
            )
        )
        documents = self.driver.find_elements_by_css_selector(
            '.fw-contents tbody tr'
        )
        self.assertEqual(
            len(documents),
            1
        )
        read_access_rights = self.driver.find_elements_by_css_selector(
            '.fw-contents tbody tr .icon-access-read'
        )
        self.assertEqual(
            len(read_access_rights),
            1
        )
        self.driver.find_element_by_css_selector(
            '.fw-contents tbody tr a.doc-title'
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located((By.CLASS_NAME, 'editor-toolbar'))
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "Some extra content that doesn't show"
        )
        assert self.driver.find_element(
            By.CSS_SELECTOR,
            ".article-title"
        ).text == "A test article to share"
        self.check_body(self.driver, 'The body')
        # Make a copy of the file
        old_body = self.driver.find_element(By.CSS_SELECTOR, ".article-body")
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".header-menu:nth-child(1) > .header-nav-item"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "li:nth-child(4) > .fw-pulldown-item"
        ).click()
        # Check whether user now has write access
        WebDriverWait(self.driver, self.wait_time).until(
            EC.staleness_of(old_body)
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "Some extra content that does show"
        )
        self.check_body(
            self.driver,
            'The bodySome extra content that does show'
        )
        # Give user 1 write access to document
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".header-menu:nth-child(1) > .header-nav-item"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "li:nth-child(1) > .fw-pulldown-item"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "#my-contacts > table > tbody > tr > td"
        ).click()
        self.driver.find_element(
            By.ID,
            "add-share-member"
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable(
                (By.CSS_SELECTOR, ".collaborator-tr .fa-caret-down")
            )
        ).click()
        self.driver.find_element_by_xpath(
            '//*[normalize-space()="Write"]'
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".ui-dialog .fw-dark"
        ).click()
        # Tag user 1 in comment
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").click()
        ActionChains(self.driver).key_down(
            Keys.SHIFT
        ).send_keys(
            Keys.LEFT
        ).send_keys(
            Keys.LEFT
        ).send_keys(
            Keys.LEFT
        ).send_keys(
            Keys.LEFT
        ).key_up(
            Keys.SHIFT
        ).perform()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "button .fa-comment"
        ).click()
        ActionChains(self.driver).send_keys(
            'Hello @Yeti'
        ).perform()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".tag-user"
        ).click()
        emails_sent_before_comment = len(mail.outbox)
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".comment-btns .submit"
        ).click()
        time.sleep(1)
        self.assertEqual(
            emails_sent_before_comment + 1,
            len(mail.outbox)
        )
        self.driver.find_element(
            By.ID,
            "close-document-top"
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.ID, 'preferences-btn'))
        ).click()
        self.driver.find_element_by_xpath(
            '//*[normalize-space()="Log out"]'
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable(
                (By.CSS_SELECTOR, 'a[href="/account/sign-up/"]')
            )
        )
        # User 4 signs up using invitation link but different email than what
        # was in the invitation email (this should work)
        invitation_link = self.find_urls(user4_invitation_email)[0]
        self.driver.get(invitation_link)
        self.driver.find_element(
            By.CSS_SELECTOR,
            'a[title="Sign up"]'
        ).click()
        self.driver.find_element(
            By.ID,
            'id_username'
        ).send_keys('Yeti4')
        self.driver.find_element(
            By.ID,
            'id_password1'
        ).send_keys('password')
        self.driver.find_element(
            By.ID,
            'id_password2'
        ).send_keys('password')
        self.driver.find_element(
            By.ID,
            'id_email'
        ).send_keys('yeti4a@snowman.com')
        self.driver.find_element(
            By.ID,
            'signup-submit'
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (
                    By.CSS_SELECTOR,
                    'a[href="mailto:yeti4a@snowman.com"]'
                )
            )
        )
        confirmation_link = self.find_urls(mail.outbox[-1].body)[0]
        self.driver.get(
            confirmation_link
        )
        self.driver.find_element(
            By.ID,
            'terms-check'
        ).click()
        self.driver.find_element(
            By.ID,
            'test-check'
        ).click()
        self.driver.find_element(
            By.ID,
            'submit'
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.text_to_be_present_in_element(
                (
                    By.CSS_SELECTOR,
                    ".fw-contents h1"
                ),
                "Thanks for verifying!"
            )
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            'a[href="/"]'
        ).click()
        self.driver.find_element(By.ID, "id_login").send_keys("Yeti4")
        self.driver.find_element(By.ID, "id_password").send_keys("password")
        self.driver.find_element(By.ID, "login-submit").click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (
                    By.CSS_SELECTOR,
                    ".new_document button"
                )
            )
        )
        documents = self.driver.find_elements_by_css_selector(
            '.fw-contents tbody tr a.doc-title'
        )
        self.assertEqual(
            len(documents),
            1
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            "#preferences-btn"
        ).click()
        self.driver.find_element_by_xpath(
            '//*[normalize-space()="Log out"]'
        ).click()
        # User 5 signs up with a different email first and then clicks the
        # invitation link. This should land user 5 directly in the editor.
        self.create_user(
            username='Yeti5',
            email='yeti5a@snowman.com',
            passtext='password'
        )
        self.driver.find_element(By.ID, "id_login").send_keys("Yeti5")
        self.driver.find_element(By.ID, "id_password").send_keys("password")
        self.driver.find_element(By.ID, "login-submit").click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (
                    By.CSS_SELECTOR,
                    ".new_document button"
                )
            )
        )
        documents = self.driver.find_elements_by_css_selector(
            '.fw-contents tbody tr a.doc-title'
        )
        self.assertEqual(
            len(documents),
            0
        )
        invitation_link = self.find_urls(user5_invitation_email)[0]
        self.driver.get(invitation_link)
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located((By.CLASS_NAME, 'editor-toolbar'))
        )
        self.driver.find_element(
            By.ID,
            "close-document-top"
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.ID, 'preferences-btn'))
        )
        documents = self.driver.find_elements_by_css_selector(
            '.fw-contents tbody tr a.doc-title'
        )
        self.assertEqual(
            len(documents),
            1
        )



from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Feedback',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('message', models.TextField()),
                ('owner', models.ForeignKey(on_delete=models.deletion.CASCADE, blank=True, to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



from allauth.account.adapter import DefaultAccountAdapter
from allauth.utils import build_absolute_uri


class AccountAdapter(DefaultAccountAdapter):

    def get_email_confirmation_url(self, request, emailconfirmation):
        """Constructs the email confirmation (activation) url.
        Note that if you have architected your system such that email
        confirmations are sent outside of the request context `request`
        can be `None` here.
        """
        url = '/account/confirm-email/{}/'.format(emailconfirmation.key)
        ret = build_absolute_uri(
            request,
            url)
        return ret

    def send_mail(self, template_prefix, email, context):
        if template_prefix == 'account/email/password_reset_key':
            # We replace the password reset URL to avoid a '/api' in the URL.
            key = context['password_reset_url'].split('/')[-2]
            url = '/account/change-password/{}/'.format(key)
            context['password_reset_url'] = build_absolute_uri(
                context['request'],
                url)
        return super().send_mail(
            template_prefix,
            email,
            context
        )





import os
from os.path import abspath, dirname, exists, join

from django.core.management.base import CommandError
import django.core.management.commands.dumpdata
import django.core.serializers
from django.db.models.fields.files import FileField
import django.dispatch
from django.core.files.storage import default_storage
from django.core.files.base import File

from fixturemedia.management.commands.loaddata import models_with_filefields


pre_dump = django.dispatch.Signal(providing_args=('instance',))


class Command(django.core.management.commands.dumpdata.Command):

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            '--outfile', dest='outfile',
            help=(
                'Specifies the file to write the serialized items to '
                '(required).'
            )
        )

    def save_images_for_signal(self, sender, **kwargs):
        instance = kwargs['instance']
        for field in sender._meta.fields:
            if not isinstance(field, FileField):
                continue
            path = getattr(instance, field.attname)
            if path is None or not path.name:
                continue

            if not default_storage.exists(path.name):
                continue

            target_path = join(self.target_dir, path.name)
            if not exists(dirname(target_path)):
                os.makedirs(dirname(target_path))

            in_file = default_storage.open(path.name, 'rb')
            file_contents = in_file.read()
            in_file.close()

            with open(target_path, 'wb') as out_file:
                out_file.write(file_contents)

    def set_up_serializer(self, ser_format):
        try:
            super_serializer = django.core.serializers.get_serializer(
                ser_format)
            super_deserializer = django.core.serializers.get_deserializer(
                ser_format)
        except KeyError:
            raise CommandError(
                "Unknown serialization format: {}".format(ser_format))

        global Serializer, Deserializer

        class Serializer(super_serializer):

            def get_dump_object(self, obj):
                pre_dump.send(sender=type(obj), instance=obj)
                return super().get_dump_object(obj)

        # We don't care about deserializing.
        Deserializer = super_deserializer

        django.core.serializers.register_serializer(
            ser_format, 'fixturemedia.management.commands.dumpdata')

    def handle(self, *app_labels, **options):
        ser_format = options.get('format')

        outfilename = options.get('outfile')
        if outfilename is None:
            raise CommandError(
                'No --outfile specified (this is a required option)')
        self.target_dir = join(dirname(abspath(outfilename)), 'media')

        for modelclass in models_with_filefields():
            pre_dump.connect(self.save_images_for_signal, sender=modelclass)

        self.set_up_serializer(ser_format)

        with File(open(outfilename, 'w')) as self.stdout:
            super().handle(*app_labels, **options)



from selenium.webdriver.common.by import By
from testing.testcases import LiveTornadoTestCase
from testing.selenium_helper import SeleniumHelper


class AccessTest(LiveTornadoTestCase, SeleniumHelper):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.base_url = cls.live_server_url
        cls.wait_time = 10

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()

    def test_ie11(self):
        driver_data = self.get_drivers(
            1,
            user_agent=(
                'Mozilla/5.0 (compatible, MSIE 11, Windows NT 6.3; '
                'Trident/7.0; rv:11.0) like Gecko'
            )
        )
        driver = driver_data["drivers"][0]
        driver.implicitly_wait(driver_data["wait_time"])
        driver.get(self.base_url + "/")
        h1 = driver.find_element(
            By.CSS_SELECTOR,
            "h1"
        )
        self.assertEqual(
            h1.text,
            'Browser not supported'
        )
        driver.quit()

    def test_robots(self):
        driver_data = self.get_drivers(
            1,
            user_agent=(
                'Mozilla/5.0 (compatible; Googlebot/2.1; '
                '+http://www.google.com/bot.html)'
            )
        )
        driver = driver_data["drivers"][0]
        driver.implicitly_wait(driver_data["wait_time"])
        driver.get(self.base_url + "/robots.txt")
        body = driver.find_element(
            By.CSS_SELECTOR,
            "body"
        )
        self.assertEqual(
            body.text,
            'User-agent: * Disallow: /* Allow: /$'
        )
        driver.quit()

    def test_hello(self):
        driver_data = self.get_drivers(1)
        driver = driver_data["drivers"][0]
        driver.implicitly_wait(driver_data["wait_time"])
        driver.get(self.base_url + "/hello-tornado")
        body = driver.find_element(
            By.CSS_SELECTOR,
            "body"
        )
        self.assertEqual(
            body.text,
            'Hello from tornado'
        )
        driver.quit()



# Generated by Django 2.2.9 on 2020-02-05 20:40

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('usermedia', '0001_squashed_0009_auto_20170908_0953'),
    ]

    operations = [
        migrations.AlterField(
            model_name='image',
            name='uploader',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='image_uploader', to=settings.AUTH_USER_MODEL),
        ),
    ]



import json
from django.db import migrations, models

def text_to_json(apps, schema_editor):
    Document = apps.get_model('document', 'Document')
    documents = Document.objects.all().iterator()
    for document in documents:
        document.content = json.loads(document.contents)
        document.diffs = json.loads(document.last_diffs)
        document.comments = json.loads(document.comments_text)
        document.bibliography = json.loads(document.bibliography_text)
        for field in document._meta.local_fields:
            if field.name == "updated":
                field.auto_now = False
        document.save()
    DocumentTemplate = apps.get_model('document', 'DocumentTemplate')
    templates = DocumentTemplate.objects.all()
    for template in templates:
        template.content = json.loads(template.definition)
        template.save()

def json_to_text(apps, schema_editor):
    Document = apps.get_model('document', 'Document')
    documents = Document.objects.all().iterator()
    for document in documents:
        document.contents = json.dumps(document.content)
        document.last_diffs = json.dumps(document.diffs)
        document.comments_text = json.dumps(document.comments)
        document.bibliography_text = json.dumps(document.bibliography)
        for field in document._meta.local_fields:
            if field.name == "updated":
                field.auto_now = False
        document.save()
    DocumentTemplate = apps.get_model('document', 'DocumentTemplate')
    templates = DocumentTemplate.objects.all()
    for template in templates:
        template.definition = json.dumps(template.content)
        template.save()

class Migration(migrations.Migration):

    dependencies = [
        ('document', '0003_fidus_3_3'),
    ]

    operations = [
        migrations.RenameField(
            model_name='document',
            old_name='comments',
            new_name='comments_text'
        ),
        migrations.AddField(
            model_name='document',
            name='comments',
            field=models.JSONField(default=dict),
        ),
        migrations.RenameField(
            model_name='document',
            old_name='bibliography',
            new_name='bibliography_text'
        ),
        migrations.AddField(
            model_name='document',
            name='bibliography',
            field=models.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name='document',
            name='content',
            field=models.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name='document',
            name='diffs',
            field=models.JSONField(default=list),
        ),
        migrations.AddField(
            model_name='documenttemplate',
            name='content',
            field=models.JSONField(default=dict),
        ),
        migrations.RunPython(text_to_json, json_to_text),
        migrations.RemoveField(
            model_name='document',
            name='comments_text',
        ),
        migrations.RemoveField(
            model_name='document',
            name='bibliography_text',
        ),
        migrations.RemoveField(
            model_name='document',
            name='contents',
        ),
        migrations.RemoveField(
            model_name='document',
            name='last_diffs',
        ),
        migrations.RemoveField(
            model_name='documenttemplate',
            name='definition',
        ),
    ]



from builtins import object

from django.db import models
from django.contrib.auth.models import User


class EntryCategory(models.Model):
    category_title = models.CharField(max_length=100)
    category_owner = models.ForeignKey(User, on_delete=models.deletion.CASCADE)

    def __str__(self):
        return self.category_title

    class Meta(object):
        verbose_name_plural = 'Entry categories'


class Entry(models.Model):
    entry_key = models.CharField(max_length=64)
    # identifier of the user, who created the entry.
    entry_owner = models.ForeignKey(User, on_delete=models.deletion.CASCADE)
    # identifier of the entrytype for the entry.
    cats = models.JSONField(default=list)
    last_modified = models.DateTimeField(auto_now=True)
    bib_type = models.CharField(max_length=30, default='')
    fields = models.JSONField(default=dict)  # all the fields

    def __str__(self):
        return self.entry_key



from .ws_handler import BaseWebSocketHandler


class WebSocket(BaseWebSocketHandler):
    sessions = dict()
    admin_sessions = dict()

    def handle_message(self, message):
        if message["type"] == 'subscribe':
            self.subscribe()
            return
        if message["type"] == 'subscribe_admin':
            self.subscribe_admin()
            return
        if message["type"] == 'message' and self.type == 'admin':
            WebSocket.send_message_to_users(message)
            self.send_message({'type': 'message_delivered'})
            return

    def subscribe(self):
        self.type = 'user'
        if len(WebSocket.sessions) == 0:
            self.id = 1
        else:
            self.id = max(WebSocket.sessions) + 1
        WebSocket.sessions[self.id] = self
        self.send_connection_info_update()

    def subscribe_admin(self):
        if not self.user.is_staff:
            # User does not have access
            self.access_denied()
            return
        self.type = 'admin'
        if len(WebSocket.admin_sessions) == 0:
            self.id = 1
        else:
            self.id = max(WebSocket.admin_sessions) + 1
        WebSocket.admin_sessions[self.id] = self
        self.send_message(self.get_connection_info_message())

    def send_connection_info_update(self):
        connection_info_message = self.get_connection_info_message()
        WebSocket.send_message_to_admins(connection_info_message)

    def get_connection_info_message(self):
        return {
            'type': 'connection_info',
            'sessions': len(WebSocket.sessions)
        }

    @classmethod
    def send_message_to_users(cls, message):
        for waiter in list(cls.sessions.values()):
            waiter.send_message(message)

    @classmethod
    def send_message_to_admins(cls, message):
        for waiter in list(cls.admin_sessions.values()):
            waiter.send_message(message)

    def on_close(self):
        if not hasattr(self, 'type'):
            return
        if self.type == 'user' and self.id in WebSocket.sessions:
            del WebSocket.sessions[self.id]
            self.send_connection_info_update()
        elif self.type == 'admin' and self.id in WebSocket.admin_sessions:
            del WebSocket.admin_sessions[self.id]



import re
import threading

from datetime import datetime
from sys import platform
from tornado.httpserver import HTTPServer
import tornado.ioloop
from tornado.web import Application

from django.core.management import call_command
from django.core.management.base import CommandError
from django.utils import translation
from django.conf import settings

from base.servers.tornado_django_hybrid import run as run_server
from base.handlers import SetupStaticFilesHandler
from base.management import BaseCommand

try:
    from asyncio import set_event_loop_policy
    from tornado.platform.asyncio import AnyThreadEventLoopPolicy
    set_event_loop_policy(AnyThreadEventLoopPolicy())
except ImportError:
    pass

naiveip_re = re.compile(r"""^(?:
(?P<addr>
    (?P<ipv4>\d{1,3}(?:\.\d{1,3}){3}) |         # IPv4 address
    (?P<ipv6>\[[a-fA-F0-9:]+\]) |               # IPv6 address
    (?P<fqdn>[a-zA-Z0-9-]+(?:\.[a-zA-Z0-9-]+)*) # FQDN
):)?(?P<port>\d+)$""", re.X)


class Command(BaseCommand):
    help = 'Run django using the tornado server'
    requires_migrations_checks = True
    requires_system_checks = True
    leave_locale_alone = True
    default_addr = '127.0.0.1'
    default_port = str(settings.PORT)
    compile_server = False

    def add_arguments(self, parser):
        parser.add_argument(
            'addrport', nargs='?',
            help='Optional port number, or ipaddr:port'
        )

    def handle(self, *args, **options):
        if options['addrport']:
            m = re.match(naiveip_re, options['addrport'])
            if m is None:
                raise CommandError(
                    '"%s" is not a valid port number '
                    'or address:port pair.' % options['addrport']
                )
            self.addr, _ipv4, _ipv6, _fqdn, self.port = m.groups()
            if not self.addr:
                self.addr = self.default_addr
        else:
            self.addr = self.default_addr
            self.port = self.default_port
        if not self.port.isdigit():
            raise CommandError("%r is not a valid port number." % self.port)
        self.inner_run(*args, **options)

    def inner_run(self, *args, **options):
        quit_command = (platform == 'win32') and 'CTRL-BREAK' or 'CONTROL-C'
        if (
            hasattr(settings, 'AUTO_SETUP') and settings.AUTO_SETUP
        ) or (
            not hasattr(settings, 'AUTO_SETUP') and settings.DEBUG
        ):
            server = self.get_setup_server()
            loop_thread = threading.Thread(
                target=tornado.ioloop.IOLoop.current().start
            )
            loop_thread.daemon = True
            loop_thread.start()
            call_command("setup", force_transpile=False)
            server.stop()
            ioloop = tornado.ioloop.IOLoop.current()
            ioloop.add_callback(ioloop.stop)
        self.stdout.write((
            "%(started_at)s\n"
            "Fidus Writer version %(version)s, using settings %(settings)r\n"
            "Fidus Writer server is running at http://%(addr)s:%(port)s/\n"
            "Quit the server with %(quit_command)s.\n"
        ) % {
            "started_at": datetime.now().strftime('%B %d, %Y - %X'),
            "version": self.get_version(),
            "settings": settings.SETTINGS_MODULE,
            "addr": self.addr,
            "port": self.port,
            "quit_command": quit_command,
        })
        # django.core.management.base forces the locale to en-us. We should
        # set it up correctly for the first request (particularly important
        # in the "--noreload" case).
        translation.activate(settings.LANGUAGE_CODE)

        run_server(self.port)

    def get_setup_server(self):
        # Start a tornado server to run while the compile is happening
        tornado_app = Application(
            [
                (
                    r'/(.*)',
                    SetupStaticFilesHandler,
                    {
                        'path': settings.SETUP_PAGE_PATH,
                        'default_filename': "index.html"
                    }
                ),
            ],
            debug=settings.DEBUG,
            websocket_ping_interval=settings.WEBSOCKET_PING_INTERVAL,
            compress_response=True
        )
        server = HTTPServer(tornado_app, no_keep_alive=True)
        server.xheaders = True
        server.listen(int(self.port))
        return server



import os

from base.management import BaseCommand
from django.core.management.utils import get_random_secret_key
from django.conf import settings


class Command(BaseCommand):
    help = (
        'Creates a new Fidus Writer project in the current directory or '
        'optionally in the given directory.'
    )

    def add_arguments(self, parser):
        parser.add_argument(
            'directory',
            nargs='?',
            help='Optional destination directory'
        )

    def handle(self, *args, **options):
        TARGET_PATH = options.pop('directory')
        SECRET_KEY = get_random_secret_key()
        if not TARGET_PATH:
            TARGET_PATH = settings.PROJECT_PATH
        if not os.path.exists(TARGET_PATH):
            os.makedirs(TARGET_PATH)
        SRC_PATH = settings.SRC_PATH
        with open(os.path.join(SRC_PATH, 'configuration.py-default')) as file:
            CONFIGURATION = file.read()
        CONFIGURATION += "\n# Don't share the SECRET_KEY with anyone."
        CONFIGURATION += "\nSECRET_KEY = '{}'".format(SECRET_KEY)
        with open(
            os.path.join(TARGET_PATH, 'configuration.py'),
            'w'
        ) as file:
            file.write(CONFIGURATION)
